"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GaslessDapp = void 0;
const web3_js_1 = require("@solana/web3.js");
const spl_token_1 = require("@solana/spl-token");
const artifacts_1 = require("./artifacts");
const token_util_1 = require("../helpers/token-util");
class GaslessDapp {
    constructor(connection, dapps) {
        this.connection = connection;
        this.dapps = dapps;
    }
    static new(connection) {
        return __awaiter(this, void 0, void 0, function* () {
            const dapps = yield (0, artifacts_1.loadArtifacts)(connection);
            return new GaslessDapp(connection, dapps);
        });
    }
    decodeTransaction(transaction) {
        let dappIxs = [];
        transaction.instructions.forEach((ix) => {
            const dapp = this.dapps.find((dapp) => ix.programId.equals(dapp.programId));
            if (dapp) {
                dappIxs.push(Object.assign({ name: dapp.name }, ix));
            }
        });
        return dappIxs;
    }
    hasDappInstruction(transaction) {
        const dappIxs = this.decodeTransaction(transaction);
        return dappIxs.length > 0 ? true : false;
    }
    addBorrowRepayForRentExemption(transaction, wallet, feePayer, rentExemption) {
        return __awaiter(this, void 0, void 0, function* () {
            const tx = new web3_js_1.Transaction();
            tx.add(web3_js_1.SystemProgram.transfer({
                fromPubkey: feePayer,
                toPubkey: wallet,
                lamports: rentExemption,
            }));
            tx.add(transaction);
            tx.add(web3_js_1.SystemProgram.transfer({
                fromPubkey: wallet,
                toPubkey: feePayer,
                lamports: rentExemption,
            }));
            return tx;
        });
    }
    build(transaction, wallet, feePayer) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.hasDappInstruction(transaction)) {
                throw Error(`dapp instruction not found`);
            }
            const newAccount = token_util_1.TokenUtil.hasInitializeNativeTokenAccountIx(transaction);
            if (newAccount) {
                const isClosed = token_util_1.TokenUtil.hasCloseTokenAccountIx(transaction, newAccount);
                const balance = yield this.connection.getBalanceAndContext(wallet);
                const rentExemption = yield spl_token_1.Token.getMinBalanceRentForExemptAccount(this.connection);
                //
                if (balance.value < rentExemption && isClosed) {
                    transaction = yield this.addBorrowRepayForRentExemption(transaction, wallet, feePayer, rentExemption);
                }
            }
            return transaction;
        });
    }
    findNeededInitAccounts(connection, transaction, gaslessFeePayer) {
        return __awaiter(this, void 0, void 0, function* () {
            const initAccounts = [];
            for (let ix of transaction.instructions) {
                const isDapp = this.dapps.find((dapp) => ix.programId.equals(dapp.programId));
                // Only pay initialize for the DApps in whitelist
                if (isDapp) {
                    const feePayer = ix.keys.find((k) => k.isSigner);
                    const keys = ix.keys.map((value) => value.pubkey);
                    const infos = yield connection.getMultipleAccountsInfo(keys);
                    if (feePayer && feePayer.pubkey.toBase58() !== gaslessFeePayer.toBase58()) {
                        for (let i = 0; i < keys.length; i++) {
                            const accountMeta = ix.keys[i];
                            const info = infos[i];
                            if (!info) {
                                initAccounts.push({
                                    newAccount: accountMeta.pubkey,
                                    feePayer: feePayer.pubkey,
                                    lamports: 0,
                                });
                            }
                        }
                    }
                }
            }
            if (initAccounts.length > 0) {
                const ixs = transaction.instructions;
                let feePayer = null;
                const grouped = groupByToMap(initAccounts, (v) => v.feePayer);
                if (grouped.size > 1) {
                    throw "Only support a single fee payer";
                }
                grouped.forEach((v, k) => {
                    feePayer = k;
                    ixs.unshift(web3_js_1.SystemProgram.transfer({
                        fromPubkey: gaslessFeePayer,
                        toPubkey: feePayer,
                        lamports: 1000000000, // 1 RENECs
                    }));
                });
                if (feePayer) {
                    transaction.instructions = ixs;
                    if (initAccounts.length > 0) {
                        const simulated = (yield this.connection.simulateTransaction(transaction, undefined, initAccounts.map((v) => v.newAccount))).value;
                        if (simulated.err) {
                            throw simulated.err;
                        }
                        if (simulated.accounts && simulated.accounts.length > 0) {
                            let totalLamports = 0;
                            for (let i = 0; i < initAccounts.length; i++) {
                                const account = simulated.accounts[i];
                                if (account) {
                                    const data = Buffer.from(account.data[0], "base64");
                                    const lamports = yield connection.getMinimumBalanceForRentExemption(data.byteLength);
                                    initAccounts[i].lamports = lamports;
                                    totalLamports += lamports;
                                }
                            }
                            const ixs = transaction.instructions;
                            ixs.shift();
                            ixs.unshift(web3_js_1.SystemProgram.transfer({
                                fromPubkey: gaslessFeePayer,
                                toPubkey: feePayer,
                                lamports: totalLamports, // 1 RENECs
                            }));
                            transaction.instructions = ixs;
                            const finalSimulated = (yield this.connection.simulateTransaction(transaction, undefined, initAccounts.map((v) => v.newAccount))).value;
                            if (finalSimulated.err) {
                                throw finalSimulated.err;
                            }
                        }
                    }
                }
            }
            return { transaction, neededInits: initAccounts };
        });
    }
}
exports.GaslessDapp = GaslessDapp;
const groupByToMap = (array, predicate) => array.reduce((map, value, index, array) => {
    var _a, _b;
    const key = predicate(value, index, array);
    (_b = (_a = map.get(key)) === null || _a === void 0 ? void 0 : _a.push(value)) !== null && _b !== void 0 ? _b : map.set(key, [value]);
    return map;
}, new Map());
