"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.postSolution = exports.getPuzzle = exports.getPendingPuzzles = exports.sendToGasless = exports.getGaslessInfo = exports.getNetwork = void 0;
const node_fetch_1 = __importDefault(require("node-fetch"));
const bs58_1 = __importDefault(require("bs58"));
const web3_js_1 = require("@solana/web3.js");
const configs_json_1 = __importDefault(require("../configs.json"));
function getNetwork(connection) {
    return __awaiter(this, void 0, void 0, function* () {
        const genesisHash = yield connection.getGenesisHash();
        const network = configs_json_1.default.find((obj) => obj.genesisHash === genesisHash);
        if (!network) {
            throw new Error(`Network is unknown. Please connect to Renec mainnet or testnet.`);
        }
        return network;
    });
}
exports.getNetwork = getNetwork;
function getGaslessInfo(connection) {
    return __awaiter(this, void 0, void 0, function* () {
        const network = yield getNetwork(connection);
        const response = yield (yield (0, node_fetch_1.default)(network.gasLessServiceURL)).json();
        if (response.feePayer && response.dapps) {
            const feePayer = new web3_js_1.PublicKey(response.feePayer);
            return { feePayer, dapps: response.dapps };
        }
        else {
            throw new Error(`getGaslessInfo error: ${response.message}`);
        }
    });
}
exports.getGaslessInfo = getGaslessInfo;
function sendToGasless(connection, signed, type, reCaptchaToken) {
    return __awaiter(this, void 0, void 0, function* () {
        const network = yield getNetwork(connection);
        const buff = signed.serialize({ requireAllSignatures: false });
        const serializedBs58 = bs58_1.default.encode(buff);
        let response;
        if (reCaptchaToken && reCaptchaToken !== "") {
            response = yield (yield (0, node_fetch_1.default)(network.gasLessServiceURL + `/v2/dapp/send`, {
                method: "POST",
                headers: { Accept: "application/json", "Content-Type": "application/json" },
                body: JSON.stringify({
                    reCaptchaToken,
                    transaction: serializedBs58,
                }),
            })).json();
        }
        else {
            response = yield (yield (0, node_fetch_1.default)(network.gasLessServiceURL + `/v1/dapp/send`, {
                method: "POST",
                headers: { Accept: "application/json", "Content-Type": "application/json" },
                body: JSON.stringify({
                    transaction: serializedBs58,
                }),
            })).json();
        }
        if (response.status === "ok") {
            const txid = response === null || response === void 0 ? void 0 : response.signature;
            return txid;
        }
        else {
            throw new Error(`sendToGasless error: ${response.message}`);
        }
    });
}
exports.sendToGasless = sendToGasless;
function getPendingPuzzles(connection, address) {
    return __awaiter(this, void 0, void 0, function* () {
        const network = yield getNetwork(connection);
        const response = yield (yield (0, node_fetch_1.default)(network.gasLessServiceURL +
            `/v1/pow/pending-puzzles?` +
            new URLSearchParams([["userAddress", address.toBase58()]]))).json();
        if (response.status === "ok") {
            const pendingPuzzles = response === null || response === void 0 ? void 0 : response.pendingPuzzles;
            return pendingPuzzles;
        }
        else {
            throw new Error(`getPendingPuzzles error: ${response.message}`);
        }
    });
}
exports.getPendingPuzzles = getPendingPuzzles;
function getPuzzle(connection, address, reCaptchaToken) {
    return __awaiter(this, void 0, void 0, function* () {
        const network = yield getNetwork(connection);
        let response;
        if (reCaptchaToken && reCaptchaToken !== "") {
            response = yield (yield (0, node_fetch_1.default)(network.gasLessServiceURL + `/v2/pow/puzzle`, {
                method: "POST",
                headers: { Accept: "application/json", "Content-Type": "application/json" },
                body: JSON.stringify({
                    reCaptchaToken,
                    userAddress: address.toBase58(),
                }),
            })).json();
        }
        else {
            response = yield (yield (0, node_fetch_1.default)(network.gasLessServiceURL +
                `/v1/pow/puzzle?` +
                new URLSearchParams([["userAddress", address.toBase58()]]))).json();
        }
        if (response.status === "ok") {
            const puzzle = response === null || response === void 0 ? void 0 : response.puzzle;
            return puzzle;
        }
        else {
            throw new Error(`getPuzzle error: ${response.message}`);
        }
    });
}
exports.getPuzzle = getPuzzle;
function postSolution(connection, rawSolution, signed) {
    return __awaiter(this, void 0, void 0, function* () {
        const network = yield getNetwork(connection);
        const buff = signed.serialize({ requireAllSignatures: false });
        const serializedBs58 = bs58_1.default.encode(buff);
        const response = yield (yield (0, node_fetch_1.default)(network.gasLessServiceURL + `/v1/pow/solution`, {
            method: "POST",
            headers: { Accept: "application/json", "Content-Type": "application/json" },
            body: JSON.stringify({
                solution: rawSolution,
                transaction: serializedBs58,
            }),
        })).json();
        if (response.status === "ok") {
            const txid = response === null || response === void 0 ? void 0 : response.signature;
            return txid;
        }
        else {
            throw new Error(`postSolution error: ${response.message}`);
        }
    });
}
exports.postSolution = postSolution;
